/*
 * Copyright (C) 1999 Silicon Graphics, Inc.  All Rights Reserved.
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of version 2 of the GNU General Public License as published
 * by the Free Software Fondation.
 * 
 * This program is distributed in the hope that it would be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  Further, any license provided herein,
 * whether implied or otherwise, is limited to this program in accordance with
 * the express provisions of the GNU General Public License.  Patent licenses,
 * if any, provided herein do not apply to combinations of this program with
 * other product or programs, or any other product whatsoever.  This program is
 * distributed without any warranty that the program is delivered free of the
 * rightful claim of any third person by way of infringement or the like.  See
 * the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston MA 02111-1307, USA.
 */

#ident "$Id: pmtrace.c,v 1.18 1999/11/18 06:56:56 kenmcd Exp $"

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <pcp/trace.h>

int
main(int argc, char **argv)
{
    int		c;
    int		state = 0;
    int		dbg;
    char	*p;
    char	*me;
    double	value;
    char	*endnum;
    int		err = 0;
    int		verbose = 1;
    char	*val = NULL;
    char	*cmd = NULL;
    char	*tag = NULL;
    char	*host = NULL;
    extern int	errno;
    extern int	optind;
    extern char	*optarg;

    /* trim command name */
    me = argv[0];
    for (p = me; *p; p++) {
	if (*p == '/')
	    me = p+1;
    }

    while ((c = getopt(argc, argv, "S:e:h:qv:?")) != EOF) {
	switch (c) {
	case 'S':
	    dbg = (int)strtol(optarg, &endnum, 10);
	    if (*endnum != '\0') {
		fprintf(stderr, "%s: -S requires a numeric argument\n", me);
		err++;
	    }
	    else
		state |= dbg;
	    break;
	case 'e':
	    if (val != NULL || cmd != NULL) {
		fprintf(stderr, "%s: only one of -v and -e is allowed\n", me);
		err++;
	    }
	    cmd = optarg;
	    break;
	case 'h':
	    host = optarg;
	    break;
	case 'q':
	    verbose = 0;
	    break;
	case 'v':
	    if (val != NULL) {
		fprintf(stderr, "%s: only one of -v and -e is allowed\n", me);
		err++;
	    }
	    val = optarg;
	    value = strtod(optarg, &endnum);
	    if (*endnum != '\0') {
		fprintf(stderr, "%s: -v requires a numeric argument\n", me);
		err++;
	    }
	    break;
	default:
	    err++;
	}
    }

    if (optind == argc-1) {
	tag = argv[optind];
	optind++;
    }

    if (err || optind != argc || tag == NULL) {
	fprintf(stderr,
"Usage: %s [-q] [-e command|-v value] [-h host] tag\n\n"
"Options:\n"
"  -e command  run command and export transaction data\n"
"  -h host     send trace data to trace PMDA on given host\n"
"  -q          quiet mode - suppress message from successful trace\n"
"  -S state    set debug state using pmtracestate(3)\n"
"  -v value    export an observation value through the trace PMDA\n", me);
	exit(0);
    }

    /* since this is a once-off operation, no need to hang around */
    pmtracestate(state | PMTRACE_STATE_ASYNC);
    state = 0;	/* reset for use as the exit status */

    if (host != NULL) {
	if ((endnum = (char *)malloc(strlen(host) + 20)) == NULL) {
	    fprintf(stderr, "%s: malloc failed: %s\n",
		    me, pmtraceerrstr(-errno));
	    exit(state);
	}
	sprintf(endnum, "PCP_TRACE_HOST=%s", host);
	if (putenv(endnum) < 0) {
	    fprintf(stderr, "%s: putenv failed: %s\n",
		    me, pmtraceerrstr(-errno));
	    exit(state);
	}
    }

    if (val != NULL) {
	if ((err = pmtraceobs(tag, value)) < 0) {
	    fprintf(stderr, "%s: observation error: %s\n",
		    me, pmtraceerrstr(err));
	    exit(state);
	}
	else if (verbose) {
	    printf("%s: observation complete (tag=\"%s\", value=%f)\n",
		   me, tag, value);
	}
    }
    else if (cmd != NULL) {
	if ((err = pmtracebegin(tag)) < 0) {
	    fprintf(stderr, "%s: transaction begin error: %s\n",
		    me, pmtraceerrstr(err));
	    /* don't exit yet - execute cmd anyway, just no tracing */
	}

	if ((state = system(cmd)) < 0) {
	    fprintf(stderr, "%s: system error: %s\n",
		    me, pmtraceerrstr(-errno));
	    state = 0;
	    exit(state);
	}
	if (err < 0)
	    exit(state);

	if ((err = pmtraceend(tag)) < 0) {
	    fprintf(stderr, "%s: transaction end error: %s\n",
		    me, pmtraceerrstr(err));
	    exit(state);
	}
	else if (verbose) {
	    printf("%s: transaction complete (tag=\"%s\")\n", me, tag);
	}
    }
    else {
	if ((err = pmtracepoint(tag)) < 0) {
	    fprintf(stderr, "%s: point error: %s\n",
		    me, pmtraceerrstr(err));
	    exit(state);
	}
	else if (verbose) {
	    printf("%s: point complete (tag=\"%s\")\n", me, tag);
	}
    }
    exit(state);
    /*NOTREACHED*/
}
